# sage_setup: distribution = sagemath-categories
r"""
Commutative rings
"""
# ****************************************************************************
#  Copyright (C) 2005      David Kohel <kohel@maths.usyd.edu>
#                          William Stein <wstein@math.ucsd.edu>
#                2008      Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#                2008-2013 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from sage.categories.category_with_axiom import CategoryWithAxiom
from sage.categories.cartesian_product import CartesianProductsCategory
from sage.structure.sequence import Sequence


class CommutativeRings(CategoryWithAxiom):
    """
    The category of commutative rings.

    commutative rings with unity, i.e. rings with commutative * and
    a multiplicative identity

    EXAMPLES::

         sage: C = CommutativeRings(); C
         Category of commutative rings
         sage: C.super_categories()
         [Category of rings, Category of commutative monoids]

    TESTS::

        sage: TestSuite(C).run()

        sage: QQ['x,y,z'] in CommutativeRings()
        True
        sage: GroupAlgebra(DihedralGroup(3), QQ) in CommutativeRings()                  # needs sage.groups sage.modules
        False
        sage: MatrixSpace(QQ, 2, 2) in CommutativeRings()                               # needs sage.modules
        False

    GroupAlgebra should be fixed::

        sage: GroupAlgebra(CyclicPermutationGroup(3), QQ) in CommutativeRings()     # not implemented, needs sage.groups sage.modules
        True
    """
    class ParentMethods:
        def is_commutative(self) -> bool:
            """
            Return whether the ring is commutative.

            The answer is ``True`` only if the category is a sub-category of
            ``CommutativeRings``.

            It is recommended to use instead ``R in Rings().Commutative()``.

            EXAMPLES::

                sage: QQ.is_commutative()
                True
                sage: QQ['x,y,z'].is_commutative()
                True
            """
            return True

        def _ideal_class_(self, n=0):
            r"""
            Return a callable object that can be used to create ideals in this
            commutative ring.

            This class can depend on `n`, the number of generators of the ideal.
            The default input of `n=0` indicates an unspecified number of generators,
            in which case a class that works for any number of generators is returned.

            EXAMPLES::

                sage: ZZ._ideal_class_()
                <class 'sage.rings.ideal.Ideal_pid'>
                sage: RR._ideal_class_()
                <class 'sage.rings.ideal.Ideal_pid'>
                sage: R.<x,y> = GF(5)[]
                sage: R._ideal_class_(1)
                <class 'sage.rings.polynomial.multi_polynomial_ideal.MPolynomialIdeal'>
                sage: S = R.quo(x^3 - y^2)
                sage: S._ideal_class_(1)
                <class 'sage.rings.quotient_ring.QuotientRingIdeal_principal'>
                sage: S._ideal_class_(2)
                <class 'sage.rings.quotient_ring.QuotientRingIdeal_generic'>
                sage: T.<z> = S[]                                                           # needs sage.libs.singular
                sage: T._ideal_class_(5)                                                    # needs sage.libs.singular
                <class 'sage.rings.ideal.Ideal_generic'>
                sage: T._ideal_class_(1)                                                    # needs sage.libs.singular
                <class 'sage.rings.ideal.Ideal_principal'>
            """
            # One might need more than just n
            from sage.rings.ideal import Ideal_generic, Ideal_principal
            return Ideal_principal if n == 1 else Ideal_generic

        def _test_divides(self, **options):
            r"""
            Run generic tests on the method :meth:`divides`.

            EXAMPLES::

                sage: ZZ._test_divides()
            """
            tester = self._tester(**options)

            # 1. is there a divides method ?
            a = self.an_element()
            try:
                a.divides
            except AttributeError:
                return

            # 2. divisibility of 0 and 1
            z = self.zero()
            o = self.one()

            tester.assertTrue(z.divides(z))
            tester.assertTrue(o.divides(o))
            tester.assertTrue(o.divides(z))
            tester.assertIs(z.divides(o), self.is_zero())

            if not self.is_exact():
                return

            # 3. divisibility of some elements
            for a, b in tester.some_elements(repeat=2):
                try:
                    test = a.divides(a * b)
                except NotImplementedError:
                    pass
                else:
                    tester.assertTrue(test)

        def over(self, base=None, gen=None, gens=None, name=None, names=None):
            r"""
            Return this ring, considered as an extension of ``base``.

            INPUT:

            - ``base`` -- a commutative ring or a morphism or ``None``
              (default: ``None``); the base of this extension or its defining
              morphism

            - ``gen`` -- a generator of this extension (over its base) or ``None``
              (default: ``None``)

            - ``gens`` -- list of generators of this extension (over its base)
              or ``None`` (default: ``None``)

            - ``name`` -- a variable name or ``None`` (default: ``None``)

            - ``names`` -- list or a tuple of variable names or ``None``
              (default: ``None``)

            EXAMPLES:

            We construct an extension of finite fields::

                sage: # needs sage.rings.finite_rings
                sage: F = GF(5^2)
                sage: k = GF(5^4)
                sage: z4 = k.gen()
                sage: K = k.over(F); K                                                  # needs sage.modules
                Field in z4 with defining polynomial
                 x^2 + (4*z2 + 3)*x + z2 over its base

            If not explicitly given, the default generator of the top ring
            (here k) is used and the same name is kept::

                sage: K.gen()                                                           # needs sage.modules sage.rings.finite_rings
                z4
                sage: K(z4)                                                             # needs sage.modules sage.rings.finite_rings
                z4

            However, it is possible to specify another generator and/or
            another name. For example::

                sage: # needs sage.modules sage.rings.finite_rings
                sage: Ka = k.over(F, name='a'); Ka
                Field in a with defining polynomial
                 x^2 + (4*z2 + 3)*x + z2 over its base
                sage: Ka.gen()
                a
                sage: Ka(z4)
                a

                sage: # needs sage.modules sage.rings.finite_rings
                sage: Kb = k.over(F, gen=-z4+1, name='b')
                sage: Kb
                Field in b with defining polynomial x^2 + z2*x + 4 over its base
                sage: Kb.gen()
                b
                sage: Kb(-z4+1)
                b

            Note that the shortcut ``K.<a>`` is also available::

                sage: KKa.<a> = k.over(F)                                               # needs sage.modules sage.rings.finite_rings
                sage: KKa is Ka                                                         # needs sage.modules sage.rings.finite_rings
                True

            Building an extension on top of another extension is allowed::

                sage: L = GF(5^12).over(K); L                                           # needs sage.modules sage.rings.finite_rings
                Field in z12 with defining polynomial
                 x^3 + (1 + (4*z2 + 2)*z4)*x^2 + (2 + 2*z4)*x - z4 over its base
                sage: L.base_ring()                                                     # needs sage.modules sage.rings.finite_rings
                Field in z4 with defining polynomial
                 x^2 + (4*z2 + 3)*x + z2 over its base

            The successive bases of an extension are accessible via the
            method :meth:`sage.rings.ring_extension.RingExtension_generic.bases`::

                sage: L.bases()                                                         # needs sage.modules sage.rings.finite_rings
                [Field in z12 with defining polynomial
                  x^3 + (1 + (4*z2 + 2)*z4)*x^2 + (2 + 2*z4)*x - z4 over its base,
                 Field in z4 with defining polynomial
                  x^2 + (4*z2 + 3)*x + z2 over its base,
                 Finite Field in z2 of size 5^2]

            When ``base`` is omitted, the canonical base of the ring is used::

                sage: S.<x> = QQ[]
                sage: E = S.over(); E                                                   # needs sage.modules
                Univariate Polynomial Ring in x over Rational Field over its base
                sage: E.base_ring()                                                     # needs sage.modules
                Rational Field

            Here is an example where ``base`` is a defining morphism::

                sage: # needs sage.modules sage.rings.number_field
                sage: k.<a> = QQ.extension(x^2 - 2)
                sage: l.<b> = QQ.extension(x^4 - 2)
                sage: f = k.hom([b^2])
                sage: L = l.over(f)
                sage: L
                Field in b with defining polynomial x^2 - a over its base
                sage: L.base_ring()
                Number Field in a with defining polynomial x^2 - 2

            Similarly, one can create a tower of extensions::

                sage: # needs sage.modules sage.rings.number_field
                sage: K = k.over()
                sage: L = l.over(Hom(K, l)(f)); L
                Field in b with defining polynomial x^2 - a over its base
                sage: L.base_ring()
                Field in a with defining polynomial x^2 - 2 over its base
                sage: L.bases()
                [Field in b with defining polynomial x^2 - a over its base,
                 Field in a with defining polynomial x^2 - 2 over its base,
                 Rational Field]
            """
            from sage.rings.ring_extension import RingExtension
            if name is not None:
                if names is not None:
                    raise ValueError("keyword argument 'name' cannot be combined with 'names'")
                names = (name,)
            if gen is not None:
                if gens is not None:
                    raise ValueError("keyword argument 'gen' cannot be combined with 'gens'")
                gens = (gen,)
            return RingExtension(self, base, gens, names)

        def frobenius_endomorphism(self, n=1):
            """
            Return the Frobenius endomorphism.

            INPUT:

            - ``n`` -- nonnegative integer (default: 1)

            OUTPUT:

            The `n`-th power of the absolute arithmetic Frobenius
            endomorphism on this commutative ring.

            EXAMPLES::

                sage: K.<u> = PowerSeriesRing(GF(5))
                sage: Frob = K.frobenius_endomorphism(); Frob
                Frobenius endomorphism x |--> x^5 of Power Series Ring in u
                 over Finite Field of size 5
                sage: Frob(u)
                u^5

            We can specify a power::

                sage: f = K.frobenius_endomorphism(2); f
                Frobenius endomorphism x |--> x^(5^2) of Power Series Ring in u
                 over Finite Field of size 5
                sage: f(1+u)
                1 + u^25
            """
            from sage.rings.morphism import FrobeniusEndomorphism_generic
            return FrobeniusEndomorphism_generic(self, n)

        def derivation_module(self, codomain=None, twist=None):
            r"""
            Return the module of derivations over this ring.

            INPUT:

            - ``codomain`` -- an algebra over this ring or a ring homomorphism
              whose domain is this ring or ``None`` (default: ``None``); if it
              is a morphism, the codomain of derivations will be the codomain
              of the morphism viewed as an algebra over ``self`` through the
              given morphism; if ``None``, the codomain will be this ring

            - ``twist`` -- a morphism from this ring to ``codomain``
              or ``None`` (default: ``None``); if ``None``, the coercion
              map from this ring to ``codomain`` will be used

            .. NOTE::

                A twisted derivation with respect to `\theta` (or a
                `\theta`-derivation for short) is an additive map `d`
                satisfying the following axiom for all `x, y` in the domain:

                .. MATH::

                    d(xy) = \theta(x) d(y) + d(x) y.

            EXAMPLES::

                sage: R.<x,y,z> = QQ[]
                sage: M = R.derivation_module(); M                                          # needs sage.modules
                Module of derivations over
                 Multivariate Polynomial Ring in x, y, z over Rational Field
                sage: M.gens()                                                              # needs sage.modules
                (d/dx, d/dy, d/dz)

            We can specify a different codomain::

                sage: K = R.fraction_field()
                sage: M = R.derivation_module(K); M                                         # needs sage.libs.singular sage.modules
                Module of derivations
                 from Multivariate Polynomial Ring in x, y, z over Rational Field
                   to Fraction Field of
                      Multivariate Polynomial Ring in x, y, z over Rational Field
                sage: M.gen() / x                                                           # needs sage.libs.singular sage.modules
                1/x*d/dx

            Here is an example with a non-canonical defining morphism::

                sage: ev = R.hom([QQ(0), QQ(1), QQ(2)])
                sage: ev
                Ring morphism:
                  From: Multivariate Polynomial Ring in x, y, z over Rational Field
                  To:   Rational Field
                  Defn: x |--> 0
                        y |--> 1
                        z |--> 2
                sage: M = R.derivation_module(ev)                                           # needs sage.modules
                sage: M                                                                     # needs sage.modules
                Module of derivations
                 from Multivariate Polynomial Ring in x, y, z over Rational Field
                   to Rational Field

            Elements in `M` acts as derivations at `(0,1,2)`::

                sage: # needs sage.modules
                sage: Dx = M.gen(0); Dx
                d/dx
                sage: Dy = M.gen(1); Dy
                d/dy
                sage: Dz = M.gen(2); Dz
                d/dz
                sage: f = x^2 + y^2 + z^2
                sage: Dx(f)  # = 2*x evaluated at (0,1,2)
                0
                sage: Dy(f)  # = 2*y evaluated at (0,1,2)
                2
                sage: Dz(f)  # = 2*z evaluated at (0,1,2)
                4

            An example with a twisting homomorphism::

                sage: theta = R.hom([x^2, y^2, z^2])
                sage: M = R.derivation_module(twist=theta); M                               # needs sage.modules
                Module of twisted derivations over Multivariate Polynomial Ring in x, y, z
                 over Rational Field (twisting morphism: x |--> x^2, y |--> y^2, z |--> z^2)

            .. SEEALSO::

                :meth:`derivation`
            """
            from sage.rings.derivation import RingDerivationModule
            if codomain is None:
                codomain = self
            return RingDerivationModule(self, codomain, twist)

        def derivation(self, arg=None, twist=None):
            r"""
            Return the twisted or untwisted derivation over this ring
            specified by ``arg``.

            .. NOTE::

                A twisted derivation with respect to `\theta` (or a
                `\theta`-derivation for short) is an additive map `d`
                satisfying the following axiom for all `x, y` in the domain:

                .. MATH::

                    d(xy) = \theta(x) d(y) + d(x) y.

            INPUT:

            - ``arg`` -- (optional) a generator or a list of coefficients
              that defines the derivation

            - ``twist`` -- (optional) the twisting homomorphism

            EXAMPLES::

                sage: R.<x,y,z> = QQ[]
                sage: R.derivation()                                                        # needs sage.modules
                d/dx

            In that case, ``arg`` could be a generator::

                sage: R.derivation(y)                                                       # needs sage.modules
                d/dy

            or a list of coefficients::

                sage: R.derivation([1,2,3])                                                 # needs sage.modules
                d/dx + 2*d/dy + 3*d/dz

            It is not possible to define derivations with respect to a
            polynomial which is not a variable::

                sage: R.derivation(x^2)                                                     # needs sage.modules
                Traceback (most recent call last):
                ...
                ValueError: unable to create the derivation

            Here is an example with twisted derivations::

                sage: R.<x,y,z> = QQ[]
                sage: theta = R.hom([x^2, y^2, z^2])
                sage: f = R.derivation(twist=theta); f                                      # needs sage.modules
                0
                sage: f.parent()                                                            # needs sage.modules
                Module of twisted derivations over Multivariate Polynomial Ring in x, y, z
                 over Rational Field (twisting morphism: x |--> x^2, y |--> y^2, z |--> z^2)

            Specifying a scalar, the returned twisted derivation is the
            corresponding multiple of `\theta - id`::

                sage: R.derivation(1, twist=theta)                                          # needs sage.modules
                [x |--> x^2, y |--> y^2, z |--> z^2] - id
                sage: R.derivation(x, twist=theta)                                          # needs sage.modules
                x*([x |--> x^2, y |--> y^2, z |--> z^2] - id)
            """
            if isinstance(arg, (list, tuple)):
                codomain = Sequence([self(0)] + list(arg)).universe()
            else:
                codomain = self
            return self.derivation_module(codomain, twist=twist)(arg)

    class ElementMethods:
        pass

    class Finite(CategoryWithAxiom):
        r"""
        Check that Sage knows that Cartesian products of finite commutative
        rings is a finite commutative ring.

        EXAMPLES::

            sage: cartesian_product([Zmod(34),
            ....:                    GF(5)]) in Rings().Commutative().Finite()
            True
        """
        def extra_super_categories(self):
            r"""
            Let Sage know that finite commutative rings are Noetherian.

            EXAMPLES::

                sage: CommutativeRings().Finite().extra_super_categories()
                [Category of noetherian rings]
            """
            from sage.categories.noetherian_rings import NoetherianRings
            return [NoetherianRings()]

        class ParentMethods:
            def cyclotomic_cosets(self, q, cosets=None):
                r"""
                Return the (multiplicative) orbits of ``q`` in the ring.

                Let `R` be a finite commutative ring. The group of invertible
                elements `R^*` in `R` gives rise to a group action on `R` by
                multiplication.  An orbit of the subgroup generated by an
                invertible element `q` is called a `q`-*cyclotomic coset* (since
                in a finite ring, each invertible element is a root of unity).

                These cosets arise in the theory of minimal polynomials of
                finite fields, duadic codes and combinatorial designs. Fix a
                primitive element `z` of `GF(q^k)`. The minimal polynomial of
                `z^s` over `GF(q)` is given by

                .. MATH::

                         M_s(x) = \prod_{i \in C_s} (x - z^i),


                where `C_s` is the `q`-cyclotomic coset mod `n` containing `s`,
                `n = q^k - 1`.

                .. NOTE::

                    When `R = \ZZ / n \ZZ` the smallest element of each coset is
                    sometimes called a *coset leader*. This function returns
                    sorted lists so that the coset leader will always be the
                    first element of the coset.

                INPUT:

                - ``q`` -- an invertible element of the ring

                - ``cosets`` -- an optional lists of elements of ``self``. If
                  provided, the function only return the list of cosets that
                  contain some element from ``cosets``.

                OUTPUT: list of lists

                EXAMPLES::

                    sage: Zmod(11).cyclotomic_cosets(2)
                    [[0], [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]]
                    sage: Zmod(15).cyclotomic_cosets(2)
                    [[0], [1, 2, 4, 8], [3, 6, 9, 12], [5, 10], [7, 11, 13, 14]]

                Since the group of invertible elements of a finite field is
                cyclic, the set of squares is a particular case of cyclotomic
                coset::

                    sage: # needs sage.rings.finite_rings
                    sage: K = GF(25, 'z')
                    sage: a = K.multiplicative_generator()
                    sage: K.cyclotomic_cosets(a**2, cosets=[1])
                    [[1, 2, 3, 4, z + 1, z + 3,
                      2*z + 1, 2*z + 2, 3*z + 3,
                      3*z + 4, 4*z + 2, 4*z + 4]]
                    sage: sorted(b for b in K if not b.is_zero() and b.is_square())
                    [1, 2, 3, 4, z + 1, z + 3,
                     2*z + 1, 2*z + 2, 3*z + 3,
                     3*z + 4, 4*z + 2, 4*z + 4]

                We compute some examples of minimal polynomials::

                    sage: # needs sage.rings.finite_rings
                    sage: K = GF(27, 'z')
                    sage: a = K.multiplicative_generator()
                    sage: R.<X> = PolynomialRing(K, 'X')
                    sage: a.minimal_polynomial('X')
                    X^3 + 2*X + 1

                    sage: cyc3 = Zmod(26).cyclotomic_cosets(3, cosets=[1]); cyc3
                    [[1, 3, 9]]
                    sage: prod(X - a**i for i in cyc3[0])                               # needs sage.rings.finite_rings
                    X^3 + 2*X + 1
                    sage: (a**7).minimal_polynomial('X')                                # needs sage.rings.finite_rings
                    X^3 + X^2 + 2*X + 1
                    sage: cyc7 = Zmod(26).cyclotomic_cosets(3, cosets=[7]); cyc7
                    [[7, 11, 21]]
                    sage: prod(X - a**i for i in cyc7[0])                               # needs sage.rings.finite_rings
                    X^3 + X^2 + 2*X + 1

                Cyclotomic cosets of fields are useful in combinatorial design
                theory to provide so called difference families (see
                :wikipedia:`Difference_set` and
                :mod:`~sage.combinat.designs.difference_family`). This is
                illustrated on the following examples::

                    sage: K = GF(5)
                    sage: a = K.multiplicative_generator()                              # needs sage.libs.pari
                    sage: H = K.cyclotomic_cosets(a**2, cosets=[1, 2]); H               # needs sage.rings.finite_rings
                    [[1, 4], [2, 3]]
                    sage: sorted(x - y for D in H for x in D for y in D if x != y)      # needs sage.rings.finite_rings
                    [1, 2, 3, 4]

                    sage: K = GF(37)
                    sage: a = K.multiplicative_generator()                              # needs sage.libs.pari
                    sage: H = K.cyclotomic_cosets(a**4, cosets=[1]); H                  # needs sage.rings.finite_rings
                    [[1, 7, 9, 10, 12, 16, 26, 33, 34]]
                    sage: sorted(x - y for D in H for x in D for y in D if x != y)      # needs sage.rings.finite_rings
                    [1, 1, 2, 2, 3, 3, 4, 4, 5, 5, ..., 33, 34, 34, 35, 35, 36, 36]

                The method ``cyclotomic_cosets`` works on any finite commutative
                ring::

                    sage: R = cartesian_product([GF(7), Zmod(14)])
                    sage: a = R((3,5))
                    sage: R.cyclotomic_cosets((3,5), [(1,1)])
                    [[(1, 1), (2, 11), (3, 5), (4, 9), (5, 3), (6, 13)]]
                """
                q = self(q)

                try:
                    ~q
                except ZeroDivisionError:
                    raise ValueError("%s is not invertible in %s" % (q, self))

                if cosets is None:
                    rest = set(self)
                else:
                    rest = {self(x) for x in cosets}

                orbits = []
                while rest:
                    x0 = rest.pop()
                    o = [x0]
                    x = q * x0
                    while x != x0:
                        o.append(x)
                        rest.discard(x)
                        x *= q
                    o.sort()
                    orbits.append(o)

                orbits.sort()
                return orbits

    class CartesianProducts(CartesianProductsCategory):
        def extra_super_categories(self):
            r"""
            Let Sage knows that Cartesian products of commutative rings is a
            commutative ring.

            EXAMPLES::

                sage: CommutativeRings().Commutative().CartesianProducts().extra_super_categories()
                [Category of commutative rings]
                sage: cartesian_product([ZZ, Zmod(34),
                ....:                    QQ, GF(5)]) in CommutativeRings()
                True
            """
            return [CommutativeRings()]



# sage_setup: distribution = sagemath-categories
r"""
Subsets satisfying a hereditary property
"""
# ****************************************************************************
#  Copyright (C) 2014 Nathann Cohen <nathann.cohen@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************


def subsets_with_hereditary_property(f, X, max_obstruction_size=None, ncpus=1):
    r"""
    Return all subsets `S` of `X` such that `f(S)` is true.

    The boolean function `f` must be decreasing, i.e. `f(S)\Rightarrow f(S')` if
    `S'\subseteq S`.

    This function is implemented to call `f` as few times as possible. More
    precisely, `f` will be called on all sets `S` such that `f(S)` is true, as
    well as on all inclusionwise minimal sets `S` such that `f(S)` is false.

    The problem that this function answers is also known as the learning problem
    on monotone boolean functions, or as computing the set of winning coalitions
    in a simple game.

    INPUT:

    - ``f`` -- boolean function which takes as input a list of elements from
      ``X``

    - ``X`` -- list/iterable

    - ``max_obstruction_size`` -- integer; if you know that there is
      a `k` such that `f(S)` is true if and only if `f(S')` is true
      for all `S'\subseteq S` with `S'\leq k`, set
      ``max_obstruction_size=k``. It may dramatically decrease the
      number of calls to `f`. Set to ``None`` by default, meaning
      `k=|X|`.

    - ``ncpus`` -- number of cpus to use for this computation. Note that
      changing the value from `1` (default) to anything different *enables*
      parallel computations which can have a cost by itself, so it is not
      necessarily a good move. In some cases, however, it is a *great* move. Set
      to ``None`` to automatically detect and use the maximum number of cpus
      available.

      .. NOTE::

          Parallel computations are performed through the
          :func:`~sage.parallel.decorate.parallel` decorator. See its
          documentation for more information, in particular with respect to the
          memory context.

    EXAMPLES:

    Sets whose elements all have the same remainder mod 2::

        sage: from sage.combinat.subsets_hereditary import subsets_with_hereditary_property
        sage: def f(x):
        ....:     return (not x) or all(xx % 2 == x[0] % 2 for xx in x)
        sage: list(subsets_with_hereditary_property(f, range(4)))
        [[], [0], [1], [2], [3], [0, 2], [1, 3]]

    Same, on two threads::

        sage: sorted(subsets_with_hereditary_property(f, range(4), ncpus=2))
        [[], [0], [0, 2], [1], [1, 3], [2], [3]]

    One can use this function to compute the independent sets of a graph. We
    know, however, that in this case the maximum obstructions are the edges, and
    have size 2. We can thus set ``max_obstruction_size=2``, which reduces the
    number of calls to `f` from 91 to 56::

        sage: # needs sage.graphs
        sage: num_calls = 0
        sage: g = graphs.PetersenGraph()
        sage: def is_independent_set(S):
        ....:     global num_calls
        ....:     num_calls += 1
        ....:     return g.subgraph(S).size() == 0
        sage: l1 = list(subsets_with_hereditary_property(is_independent_set,
        ....:                                            g.vertices(sort=False)))
        sage: num_calls
        91
        sage: num_calls = 0
        sage: l2 = list(subsets_with_hereditary_property(is_independent_set,
        ....:                                            g.vertices(sort=False),
        ....:                                            max_obstruction_size=2))
        sage: num_calls
        56
        sage: l1 == l2
        True

    TESTS::

        sage: list(subsets_with_hereditary_property(lambda x: False, range(4)))
        []
        sage: list(subsets_with_hereditary_property(lambda x: len(x)<1, range(4)))
        [[]]
        sage: list(subsets_with_hereditary_property(lambda x: True, range(2)))
        [[], [0], [1], [0, 1]]
    """
    from sage.data_structures.bitset import Bitset
    # About the implementation:
    #
    # 1) We work on X={0,...,n-1} but remember X to return correctly
    #    labelled answers.
    #
    # 2) We maintain a list of sets S such that f(S)=0 (i.e. 'no-sets'), in
    #    order to filter out larger sets for which f is necessarily False.
    #
    # 3) Those sets are stored in an array: bs[i] represents the set of all
    #    no-sets S we found such that i is NOT in S. Why ? Because it makes it
    #    easy to filter out sets: if a set S' whose *complement* is
    #    {i1,i2,...,ik} is such that bs[i1]&bs[i2]&...&bs[ik] is nonempty then
    #    f(S') is necessarily False.
    X_labels = list(X)
    n = len(X_labels)
    X = set(range(n))
    if max_obstruction_size is None:
        max_obstruction_size = n

    bs = [Bitset([], 1) for _ in range(n)]  # collection of no-set
    nforb = 1                              # number of no-sets stored
    current_layer = [[]]                  # all yes-sets of size 'current_size'
    current_size = 0

    def explore_neighbors(s):
        r"""
        Explore the successors of a set s.

        The successors of a set s are all the sets s+[i] where max(s)<i. This
        function returns them all as a partition `(yes_sets,no_sets)`.
        """
        new_yes_sets = []
        new_no_sets = []
        for i in range((s[-1] + 1 if s else 0), n):        # all ways to extend it
            s_plus_i = s + [i]                             # the extended set
            s_plus_i_c = Bitset(s_plus_i, n).complement()  # .. and its complement

            # Filter a no-set using the data collected so far.
            inter = Bitset([], nforb).complement()
            for j in s_plus_i_c:
                inter.intersection_update(bs[j])

            # If we cannot decide yet we must call f(S)
            if not inter:
                if set_size >= max_obstruction_size or f([X_labels[xx] for xx in s_plus_i]):
                    new_yes_sets.append(s_plus_i)
                else:
                    new_no_sets.append(s_plus_i)
        return (new_yes_sets, new_no_sets)

    # The empty set
    if f([]):
        yield []
    else:
        return

    if ncpus != 1:
        from sage.parallel.decorate import parallel
        explore_neighbors_paral = parallel(ncpus=ncpus)(explore_neighbors)

    # All sets of size 0, then size 1, then ...
    set_size = -1
    while current_layer:
        set_size += 1
        new_no_sets = []
        new_yes_sets = []

        if ncpus == 1:
            yes_no_iter = (explore_neighbors(s) for s in current_layer)
        else:
            yes_no_iter = ((yes, no) for (_, (yes, no)) in explore_neighbors_paral(current_layer))

        for yes, no in yes_no_iter:
            new_yes_sets.extend(yes)
            new_no_sets.extend(no)
            for s in yes:
                yield [X_labels[xx] for xx in s]

        current_layer = new_yes_sets

        # Update bs with the new no-sets
        new_nforb = nforb + len(new_no_sets)
        for b in bs:  # resize the bitsets
            b.add(new_nforb)
            b.discard(new_nforb)
        for i, s in enumerate(new_no_sets):  # Fill the new entries
            for j in X.difference(s):
                bs[j].add(i + nforb)
        nforb = new_nforb
        current_size += 1

    # Did we forget to return X itself ?
    #
    # If we did, this was probably the worst choice of algorithm for we computed
    # f(X) for all 2^n sets X, but well...
    if (current_size == len(X) and nforb == 1 and f(X_labels)):
        yield X_labels



# sage_setup: distribution = sagemath-categories
"""
HTML Fragments

This module defines a HTML fragment class, which holds a piece of
HTML. This is primarily used in browser-based notebooks, though it
might be useful for creating static pages as well.

This module defines :class:`MathJax`, an object of which performs the task of
producing an HTML representation of any object. The produced HTML is
renderable in a browser-based notebook with the help of MathJax.
"""

# *****************************************************************************
#       Copyright (C) 2008 William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
# *****************************************************************************

import re

from sage.misc.latex import latex
from sage.structure.sage_object import SageObject

macro_regex = re.compile(r'\\newcommand{(?P<name>\\[a-zA-Z]+)}(\[.+\])?{(?P<definition>.+)}')


class HtmlFragment(str, SageObject):
    r"""
    A HTML fragment.

    This is a piece of HTML, usually not a complete document.  For
    example, just a ``<div>...</div>`` piece and not the entire
    ``<html>...</html>``.

    EXAMPLES::

        sage: from sage.misc.html import HtmlFragment
        sage: HtmlFragment('<b>test</b>')
        <b>test</b>

    .. automethod:: _rich_repr_
    """

    def _rich_repr_(self, display_manager, **kwds):
        """
        Rich Output Magic Method.

        See :mod:`sage.repl.rich_output` for details.

        EXAMPLES::

            sage: from sage.repl.rich_output import get_display_manager
            sage: dm = get_display_manager()
            sage: h = sage.misc.html.HtmlFragment('<b>old</b>')
            sage: h._rich_repr_(dm)  # the doctest backend does not support html
            OutputPlainText container
        """
        OutputHtml = display_manager.types.OutputHtml
        if OutputHtml in display_manager.supported_output():
            return OutputHtml(self)
        else:
            return display_manager.types.OutputPlainText(self)


def math_parse(s):
    r"""
    Transform the string ``s`` with TeX maths to an HTML string renderable by
    MathJax.

    INPUT:

    - ``s`` -- string

    OUTPUT: :class:`HtmlFragment`

    Specifically this method does the following:

    * Replace all ``\$text\$``\'s by ``\(text\)``
    * Replace all ``\$\$text\$\$``\'s by ``\[text\]``
    * Replace all ``\\\$``\'s by ``\$``\'s. Note that this has precedence over
      the above two cases.

    EXAMPLES::

        sage: print(sage.misc.html.math_parse('This is $2+2$.'))
        This is \(2+2\).
        sage: print(sage.misc.html.math_parse('This is $$2+2$$.'))
        This is \[2+2\].
        sage: print(sage.misc.html.math_parse('This is \\[2+2\\].'))
        This is \[2+2\].
        sage: print(sage.misc.html.math_parse(r'\$2+2\$ is rendered to $2+2$.'))
        <span>$</span>2+2<span>$</span> is rendered to \(2+2\).
    """
    # Below t always has the "parsed so far" version of s, and s is
    # just the part of the original input s that hasn't been parsed.
    t = ''
    while True:
        i = s.find('$')
        if i == -1:
            # No dollar signs -- definitely done.
            return HtmlFragment(t + s)
        elif i > 0 and s[i-1] == '\\':
            # A dollar sign with a backslash right before it, so this is a
            # normal dollar sign. If processEscapes is enabled in MathJax, "\$"
            # will do the job. But as we do not assume that, we use the span
            # tag safely.
            t += s[:i-1] + '<span>$</span>'
            s = s[i+1:]
            continue
        elif i+1 < len(s) and s[i+1] == '$':
            # Found a math environment. Double dollar sign so display mode.
            disp = True
        else:
            # Found math environment. Single dollar sign so default mode.
            disp = False

        # Now find the matching $ sign and form the html string.

        if disp:
            j = s[i+2:].find('$$')
            if j == -1:
                j = len(s)
                s += '$$'
            else:
                j += i + 2
            txt = s[i+2:j]
        else:
            j = s[i+2:].find('$')
            if j == -1:
                j = len(s)
                s += '$'
            else:
                j += i + 2
            txt = s[i+1:j]

        if disp:
            t += s[:i] + r'\[{0}\]'.format(' '.join(txt.splitlines()))
        else:
            t += s[:i] + r'\({0}\)'.format(' '.join(txt.splitlines()))
        s = s[j+1:]
        if disp:
            s = s[1:]
    return HtmlFragment(t)


class MathJaxExpr:
    """
    An arbitrary MathJax expression that can be nicely concatenated.

    EXAMPLES::

        sage: from sage.misc.html import MathJaxExpr
        sage: MathJaxExpr("a^{2}") + MathJaxExpr("x^{-1}")
        a^{2}x^{-1}
    """
    def __init__(self, y):
        """
        Initialize a MathJax expression.

        INPUT:

        - ``y`` -- string

        Note that no error checking is done on the type of ``y``.

        EXAMPLES::

            sage: from sage.misc.html import MathJaxExpr
            sage: jax = MathJaxExpr(3); jax  # indirect doctest
            3
            sage: TestSuite(jax).run(skip ='_test_pickling')
        """
        self.__y = y

    def __repr__(self):
        """
        Print representation.

        EXAMPLES::

            sage: from sage.misc.html import MathJaxExpr
            sage: jax = MathJaxExpr('3')
            sage: jax.__repr__()
            '3'
        """
        return str(self.__y)

    def __add__(self, y):
        """
        'Add' MathJaxExpr ``self`` to ``y``.  This concatenates them
        (assuming that they're strings).

        EXAMPLES::

            sage: from sage.misc.html import MathJaxExpr
            sage: j3 = MathJaxExpr('3')
            sage: jx = MathJaxExpr('x')
            sage: j3 + jx
            3x
        """
        return MathJaxExpr(self.__y + y)

    def __radd__(self, y):
        """
        'Add' MathJaxExpr ``y`` to ``self``.  This concatenates them
        (assuming that they're strings).

        EXAMPLES::

            sage: from sage.misc.html import MathJaxExpr
            sage: j3 = MathJaxExpr('3')
            sage: jx = MathJaxExpr('x')
            sage: j3.__radd__(jx)
            x3
        """
        return MathJaxExpr(y + self.__y)


class MathJax:
    r"""
    Render LaTeX input using MathJax.  This returns a :class:`MathJaxExpr`.

    EXAMPLES::

        sage: from sage.misc.html import MathJax
        sage: MathJax()(3)
        <html>\[3\]</html>
        sage: MathJax()(ZZ)
        <html>\[\newcommand{\Bold}[1]{\mathbf{#1}}\Bold{Z}\]</html>
    """

    def __call__(self, x, combine_all=False):
        r"""
        Render LaTeX input using MathJax.  This returns a :class:`MathJaxExpr`.

        INPUT:

        - ``x`` -- a Sage object

        - ``combine_all`` -- boolean (default: ``False``); if ``combine_all``
          is ``True`` and the input is a tuple, then it does not return a tuple
          and instead returns a string with all the elements separated by
          a single space

        OUTPUT: :class:`MathJaxExpr`

        EXAMPLES::

            sage: from sage.misc.html import MathJax
            sage: MathJax()(3)
            <html>\[3\]</html>
            sage: str(MathJax().eval(ZZ['x'], mode='display')) == str(MathJax()(ZZ['x']))
            True
        """
        return self.eval(x, combine_all=combine_all)

    def eval(self, x, globals=None, locals=None, mode='display', combine_all=False):
        r"""
        Render LaTeX input using MathJax.  This returns a :class:`MathJaxExpr`.

        INPUT:

        - ``x`` -- a Sage object

        - ``globals`` -- a globals dictionary

        - ``locals`` -- extra local variables used when
          evaluating Sage code in ``x``

        - ``mode`` -- string (default: ``'display'``);
          ``'display'`` for displaymath, ``'inline'`` for inline
          math, or ``'plain'`` for just the LaTeX code without the
          surrounding html and script tags

        - ``combine_all`` -- boolean (default: ``False``); if ``combine_all``
          is ``True`` and the input is a tuple, then it does not return a tuple
          and instead returns a string with all the elements separated by
          a single space

        OUTPUT: :class:`MathJaxExpr`

        EXAMPLES::

            sage: from sage.misc.html import MathJax
            sage: MathJax().eval(3, mode='display')
            <html>\[3\]</html>
            sage: MathJax().eval(3, mode='inline')
            <html>\(3\)</html>
            sage: MathJax().eval(type(3), mode='inline')
            <html>\(\verb|&lt;class|\verb| |\verb|'sage.rings.integer.Integer'>|\)</html>

        TESTS:

            sage: from sage.misc.html import MathJax
            sage: MathJax().eval(IntegerModRing(6))
            <html>\[\newcommand{\ZZ}{\Bold{Z}}\newcommand{\Bold}[1]{\mathbf{#1}}\ZZ/6\ZZ\]</html>
        """
        # Get a regular LaTeX representation of x
        x = latex(x, combine_all=combine_all)

        # The "\text{\texttt{...}}" blocks are reformed to be renderable by MathJax.
        # These blocks are produced by str_function() defined in sage.misc.latex.
        prefix = r"\text{\texttt{"
        parts = x.split(prefix)
        for i, part in enumerate(parts):
            if i == 0:
                continue    # Nothing to do with the head part
            n = 1
            for closing, c in enumerate(part):
                if c == "{" and part[closing - 1] != "\\":
                    n += 1
                if c == "}" and part[closing - 1] != "\\":
                    n -= 1
                if n == -1:
                    break
            # part should end in "}}", so omit the last two characters
            # from y
            y = part[:closing-1]
            for delimiter in r"""|"'`#%&,.:;?!@_~^+-/\=<>()[]{}0123456789E""":
                if delimiter not in y:
                    break
            if delimiter == "E":
                # y is too complicated
                delimiter = "|"
                y = "(complicated string)"
            wrapper = r"\verb" + delimiter + "%s" + delimiter
            y = y.replace("{ }", " ").replace("{-}", "-")
            for c in r"#$%&\^_{}~":
                char_wrapper = r"{\char`\%s}" % c
                y = y.replace(char_wrapper, c)
            subparts = []
            nspaces = 0
            for subpart in y.split(" "):
                if subpart == "":
                    nspaces += 1
                    continue
                if nspaces > 0:
                    subparts.append(wrapper % (" " * nspaces))
                nspaces = 1
                subparts.append(wrapper % subpart)
            subparts.append(part[closing + 1:])
            parts[i] = "".join(subparts)

        from sage.misc.latex_macros import sage_latex_macros
        from sage.misc.latex import _Latex_prefs

        latex_string = ''.join([_Latex_prefs._option['macros']] + parts)

        # add a macro definition only if it appears in the latex string
        macros_string = ''
        latex_macros = sage_latex_macros()
        test_string = latex_string
        while test_string:
            new_test_string = ''
            for line in latex_macros:
                m = macro_regex.match(line)
                if m['name'] in test_string:
                    macros_string += line
                    new_test_string += '{' + m['definition'] + '}'
            test_string = new_test_string

        latex_string = macros_string + latex_string

        mathjax_string = latex_string.replace('<', '&lt;')
        if mode == 'display':
            html = r'<html>\[{0}\]</html>'
        elif mode == 'inline':
            html = r'<html>\({0}\)</html>'
        elif mode == 'display_left':
            html = r'<html>\(\displaystyle {0}\)</html>'
        elif mode == 'plain':
            return mathjax_string
        else:
            raise ValueError("mode must be either 'display', 'inline', 'display_left' or 'plain'")
        return MathJaxExpr(html.format(mathjax_string))


class HTMLFragmentFactory(SageObject):

    def _repr_(self):
        """
        Return string representation.

        OUTPUT: string

        EXAMPLES::

            sage: html
            Create HTML output (see html? for details)
        """
        return 'Create HTML output (see html? for details)'

    def __call__(self, obj, concatenate=True, strict=False):
        r"""
        Construct a HTML fragment.

        INPUT:

        - ``obj`` -- anything. An object for which you want an HTML
          representation

        - ``concatenate`` -- if ``True``, combine HTML representations of
          elements of the container ``obj``

        - ``strict`` -- if ``True``, construct an HTML representation of
          ``obj`` even if ``obj`` is a string

        OUTPUT: :class:`HtmlFragment`

        EXAMPLES::

            sage: h = html('<hr>');  pretty_print(h)
            <hr>
            sage: type(h)
            <class 'sage.misc.html.HtmlFragment'>

            sage: html(1/2)
            <html>\(\displaystyle \frac{1}{2}\)</html>

            sage: html('<a href="http://sagemath.org">sagemath</a>')
            <a href="http://sagemath.org">sagemath</a>

            sage: html('<a href="http://sagemath.org">sagemath</a>', strict=True)
            <html>\(\displaystyle \verb|&lt;a|\verb| |\verb|href="http://sagemath.org">sagemath&lt;/a>|\)</html>

        Display preference ``align_latex`` affects rendering of LaTeX expressions::

            sage: from sage.repl.rich_output.display_manager import get_display_manager
            sage: dm = get_display_manager()
            sage: dm.preferences.align_latex = 'left'
            sage: html(1/2)
            <html>\(\displaystyle \frac{1}{2}\)</html>
            sage: dm.preferences.align_latex = 'center'
            sage: html(1/2)
            <html>\[\frac{1}{2}\]</html>
            sage: dm.preferences.align_latex = None  # same with left
            sage: html(1/2)
            <html>\(\displaystyle \frac{1}{2}\)</html>
        """
        # string obj is interpreted as an HTML in not strict mode
        if isinstance(obj, str) and not strict:
            return HtmlFragment(math_parse(obj))

        # prefer dedicated _html_() method
        try:
            result = obj._html_()
            return HtmlFragment(result)
        except AttributeError:
            pass

        from sage.repl.rich_output.display_manager import get_display_manager
        dm = get_display_manager()
        if dm.preferences.align_latex == 'center':
            mode = 'display'
        elif dm.preferences.align_latex == 'left':
            mode = 'display_left'
        else:
            mode = 'display_left'

        # otherwise convert latex to html
        if concatenate:
            if isinstance(obj, (tuple, list)):
                obj = tuple(obj)
            result = MathJax().eval(obj, mode=mode, combine_all=True)
        else:
            result = MathJax().eval(obj, mode=mode, combine_all=False)
        return HtmlFragment(result)

    def eval(self, s, locals=None):
        r"""
        Evaluate embedded <sage> tags.

        INPUT:

        - ``s`` -- string

        - ``globals`` -- dictionary; the global variables when
          evaluating ``s``. Default: the current global variables.

        OUTPUT: :class:`HtmlFragment`

        EXAMPLES::

            sage: a = 123
            sage: html.eval('<sage>a</sage>')
            \(123\)
            sage: html.eval('<sage>a</sage>', locals={'a': 456})
            \(456\)
        """
        from sage.misc.sage_eval import sage_eval
        if locals is None:
            from sage.repl.user_globals import get_globals
            locals = get_globals()
        s = str(s)
        s = math_parse(s)
        t = ''
        while s:
            i = s.find('<sage>')
            if i == -1:
                t += s
                break
            j = s.find('</sage>')
            if j == -1:
                t += s
                break
            t += s[:i] + r'\({}\)'.format(latex(sage_eval(s[6+i:j], locals=locals)))
            s = s[j+7:]
        return HtmlFragment(t)

    def iframe(self, url, height=400, width=800):
        r"""
        Generate an iframe HTML fragment.

        INPUT:

        - ``url`` -- string; a url, either with or without URI scheme
          (defaults to "http"), or an absolute file path

        - ``height`` -- the number of pixels for the page height
          Defaults to 400

        - ``width`` -- the number of pixels for the page width
          Defaults to 800

        OUTPUT: :class:`HtmlFragment`

        EXAMPLES::

            sage: pretty_print(html.iframe("sagemath.org"))
            <iframe height="400" width="800"
            src="http://sagemath.org"></iframe>
            sage: pretty_print(html.iframe("http://sagemath.org",30,40))
            <iframe height="30" width="40"
            src="http://sagemath.org"></iframe>
            sage: pretty_print(html.iframe("https://sagemath.org",30))
            <iframe height="30" width="800"
            src="https://sagemath.org"></iframe>
            sage: pretty_print(html.iframe("/home/admin/0/data/filename"))
            <iframe height="400" width="800"
            src="file:///home/admin/0/data/filename"></iframe>
            sage: pretty_print(html.iframe('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAA'
            ....: 'AUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBA'
            ....: 'AO9TXL0Y4OHwAAAABJRU5ErkJggg=="'))
            <iframe height="400" width="800"
            src="http://data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg==""></iframe>
        """
        if url.startswith('/'):
            url = 'file://{0}'.format(url)
        elif '://' not in url:
            url = 'http://{0}'.format(url)
        return HtmlFragment('<iframe height="{0}" width="{1}" src="{2}"></iframe>'
                            .format(height, width, url))


html = HTMLFragmentFactory()
html.__doc__ = HTMLFragmentFactory.__call__.__doc__


def pretty_print_default(enable=True):
    r"""
    Enable or disable default pretty printing.

    Pretty printing means rendering things in HTML and by MathJax so that a
    browser-based frontend can render real math.

    This function is pretty useless without the notebook, it should not
    be in the global namespace.

    INPUT:

    - ``enable`` -- boolean (default: ``True``);  if ``True``, turn on
      pretty printing. If ``False``, turn it off.

    EXAMPLES::

        sage: pretty_print_default(True)
        sage: 'foo'  # the doctest backend does not support html
        'foo'
        sage: pretty_print_default(False)
        sage: 'foo'
        'foo'
    """
    from sage.repl.rich_output import get_display_manager
    dm = get_display_manager()
    dm.preferences.text = 'latex' if enable else None



# sage_setup: distribution = sagemath-categories
# sage.doctest: needs sage.libs.pari
"""
Basic arithmetic with C integers
"""

# ****************************************************************************
#       Copyright (C) 2004 William Stein <wstein@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

###################################################################
# We define the following functions in this file, both
# for int (up to bound = 2**31 - 1) and longlong (up to 2**63 - 1).
# The function definitions are identical except for the types.
# Some of their input can be at most sqrt(bound), since
# it is necessary to multiply numbers and reduce the product
# modulo n, where n is at most bound.
#
#   * abs_int -- absolute value of integer
#   * sign_int -- sign of integer
#   * c_gcd_int -- gcd of two ints
#   * gcd_int -- python export of c_gcd_int
#   * c_xgcd_int -- extended gcd of two ints
#   * c_inverse_mod_int -- inverse of an int modulo another int
#   * c_rational_recon_int -- rational reconstruction of ints
#   * rational_recon_int -- export of rational reconstruction for ints
#
#  The long long functions are the same, except they end in _longlong.
#
###################################################################

# The int definitions

from libc.math cimport sqrt

from sage.rings.integer cimport Integer

cpdef prime_range(start, stop=None, algorithm=None, bint py_ints=False):
    r"""
    Return a list of all primes between ``start`` and ``stop - 1``, inclusive.

    If the second argument is omitted, this returns the primes up to the
    first argument.

    The sage command :func:`~sage.arith.misc.primes` is an alternative that
    uses less memory (but may be slower), because it returns an iterator,
    rather than building a list of the primes.

    INPUT:

    - ``start`` -- integer; lower bound (default: 1)

    - ``stop`` -- integer; upper bound

    - ``algorithm`` -- string (default: ``None``), one of:

      - ``None``: Use  algorithm ``'pari_primes'`` if ``stop`` <= 436273009
        (approximately 4.36E8). Otherwise use algorithm ``'pari_isprime'``.

      - ``'pari_primes'``: Use PARI's :pari:`primes` function to generate all
        primes from 2 to stop. This is fast but may crash if there is
        insufficient memory. Raises an error if ``stop`` > 436273009.

      - ``'pari_isprime'``: Wrapper for ``list(primes(start, stop))``. Each (odd)
        integer in the specified range is tested for primality by applying PARI's
        :pari:`isprime` function. This is slower but will work for much larger input.

    - ``py_ints`` -- boolean (default: ``False``); return Python ints rather
      than Sage Integers (faster). Ignored unless algorithm ``'pari_primes'`` is being
      used.

    EXAMPLES::

        sage: prime_range(10)
        [2, 3, 5, 7]
        sage: prime_range(7)
        [2, 3, 5]
        sage: prime_range(2000,2020)
        [2003, 2011, 2017]
        sage: prime_range(2,2)
        []
        sage: prime_range(2,3)
        [2]
        sage: prime_range(5,10)
        [5, 7]
        sage: prime_range(-100,10,"pari_isprime")
        [2, 3, 5, 7]
        sage: prime_range(2,2,algorithm='pari_isprime')
        []
        sage: prime_range(10**16,10**16+100,"pari_isprime")
        [10000000000000061, 10000000000000069, 10000000000000079, 10000000000000099]
        sage: prime_range(10**30,10**30+100,"pari_isprime")
        [1000000000000000000000000000057, 1000000000000000000000000000099]
        sage: type(prime_range(8)[0])
        <class 'sage.rings.integer.Integer'>
        sage: type(prime_range(8,algorithm='pari_isprime')[0])
        <class 'sage.rings.integer.Integer'>

    .. NOTE::

        ``start`` and ``stop`` should be integers, but real numbers will also be accepted
        as input. In this case, they will be rounded to nearby integers start\* and
        stop\*, so the output will be the primes between start\* and stop\* - 1, which may
        not be exactly the same as the primes between ``start`` and ``stop - 1``.

    TESTS::

        sage: prime_range(-1)
        []
        sage: L = prime_range(25000,2500000)
        sage: len(L)
        180310
        sage: L[-10:]
        [2499923, 2499941, 2499943, 2499947, 2499949, 2499953, 2499967, 2499983, 2499989, 2499997]

    A non-trivial range without primes::

        sage: prime_range(4652360, 4652400)
        []

    Test for non-existing algorithm::

        sage: prime_range(55, algorithm='banana')
        Traceback (most recent call last):
        ...
        ValueError: algorithm must be "pari_primes" or "pari_isprime"

    Confirm the fixes for :issue:`28467`::

        sage: prime_range(436273009, 436273010)
        [436273009]
        sage: prime_range(436273009, 436273010, algorithm='pari_primes')
        Traceback (most recent call last):
        ...
        ValueError: algorithm "pari_primes" is limited to primes larger than 436273008

    AUTHORS:

    - William Stein (original version)
    - Craig Citro (rewrote for massive speedup)
    - Kevin Stueve (added primes iterator option) 2010-10-16
    - Robert Bradshaw (speedup using Pari prime table, py_ints option)
    """
    # input to pari.init_primes cannot be greater than 436273290 (hardcoded bound)
    DEF init_primes_max = 436273290
    DEF small_prime_max = 436273009  # a prime < init_primes_max (preferably the largest)
    DEF prime_gap_bound = 250        # upper bound for gap between primes <= small_prime_max

    # make sure that start and stop are integers
    # First try coercing them. If that does not work, then try rounding them.
    try:
        start = Integer(start)
    except TypeError as integer_error:
        try:
            start = Integer(round(float(start)))
        except (ValueError, TypeError) as real_error:
            raise TypeError(str(integer_error)
                            + "\nand argument is also not real: "
                            + str(real_error))
    if stop is not None:
        try:
            stop = Integer(stop)
        except TypeError as integer_error:
            try:
                stop = Integer(round(float(stop)))
            except (ValueError, TypeError) as real_error:
                raise ValueError(str(integer_error)
                                 + "\nand argument is also not real: "
                                 + str(real_error))

    if algorithm is None:
        # if 'stop' is 'None', need to change it to an integer before comparing with 'start'
        if max(start, stop or 0) <= small_prime_max:
            algorithm = "pari_primes"
        else:
            algorithm = "pari_isprime"

    if algorithm == "pari_primes":
        from sage.libs.pari.convert_sage import pari_maxprime, pari_prime_range
        from sage.libs.pari import pari

        if max(start, stop or 0) > small_prime_max:
            raise ValueError('algorithm "pari_primes" is limited to primes '
                             f'larger than {small_prime_max - 1}')

        if stop is None:
            # In this case, "start" is really stop
            stop = start
            start = 1
        else:
            start = start
            stop = stop
            if start < 1:
                start = 1
        if stop <= start:
            return []

        if pari_maxprime() < stop:
            # Adding prime_gap_bound should be sufficient to guarantee an
            # additional prime, given that c_stop <= small_prime_max.
            pari.init_primes(min(stop + prime_gap_bound, init_primes_max))
            assert pari_maxprime() >= stop

        res = pari_prime_range(start, stop, py_ints)

    elif algorithm == "pari_isprime" or algorithm == "pari_primes":
        from sage.arith.misc import primes
        res = list(primes(start, stop))
    else:
        raise ValueError('algorithm must be "pari_primes" or "pari_isprime"')
    return res


cdef class arith_int:
    cdef int abs_int(self, int x) except -1:
        if x < 0:
            return -x
        return x

    cdef int sign_int(self, int n) except -2:
        if n < 0:
            return -1
        return 1

    cdef int c_gcd_int(self, int a, int b) except -1:
        cdef int c
        if a == 0:
            return self.abs_int(b)
        if b == 0:
            return self.abs_int(a)
        if a < 0:
            a = -a
        if b < 0:
            b = -b
        while b:
            c = a % b
            a = b
            b = c
        return a

    def gcd_int(self, int a, int b):
        return self.c_gcd_int(a, b)

    cdef int c_xgcd_int(self, int a, int b, int* ss, int* tt) except -1:
        cdef int psign, qsign, p, q, r, s, c, quot, new_r, new_s

        if a == 0:
            ss[0] = 0
            tt[0] = self.sign_int(b)
            return self.abs_int(b)

        if b == 0:
            ss[0] = self.sign_int(a)
            tt[0] = 0
            return self.abs_int(a)

        psign = 1
        qsign = 1

        if a < 0:
            a = -a
            psign = -1
        if b < 0:
            b = -b
            qsign = -1

        p = 1
        q = 0
        r = 0
        s = 1
        while b:
            c = a % b
            quot = a / b
            a = b
            b = c
            new_r = p - quot * r
            new_s = q - quot * s
            p = r
            q = s
            r = new_r
            s = new_s

        ss[0] = p * psign
        tt[0] = q * qsign

        return a

    def xgcd_int(self, int a, int b):
        cdef int g, s, t
        g = self.c_xgcd_int(a, b, &s, &t)
        return (g, s, t)

    cdef int c_inverse_mod_int(self, int a, int m) except -1:
        if a == 1 or m <= 1:
            return a % m   # common special case
        cdef int g, s, t
        g = self.c_xgcd_int(a, m, &s, &t)
        if g != 1:
            raise ArithmeticError("The inverse of %s modulo %s is not defined." % (a, m))
        s = s % m
        if s < 0:
            s = s + m
        return s

    def inverse_mod_int(self, int a, int m):
        return self.c_inverse_mod_int(a, m)

    cdef int c_rational_recon_int(self, int a, int m, int* n, int* d) except -1:
        cdef int u, v, u0, u1, u2, v0, v1, v2, q, t0, t1, t2, x, y
        cdef float bnd

        if m > 46340:
            raise OverflowError(f"The modulus m(={m}) should be at most 46340")

        a = a % m

        if a == 0 or m == 0:
            n[0] = 0
            d[0] = 1
            return 0

        if m < 0:
            m = -m
        if a < 0:
            a = m - a
        if a == 1:
            n[0] = 1
            d[0] = 1
            return 0

        u = m
        v = a
        bnd = sqrt(m/2.0)
        u0 = 1
        u1 = 0
        u2 = u
        v0 = 0
        v1 = 1
        v2 = v
        while self.abs_int(v2) > bnd:
            q = u2 / v2   # floor is implicit
            t0 = u0 - q * v0
            t1 = u1 - q * v1
            t2 = u2 - q * v2
            u0 = v0
            u1 = v1
            u2 = v2
            v0 = t0
            v1 = t1
            v2 = t2

        x = self.abs_int(v1)
        y = v2
        if v1 < 0:
            y = -1*y
        if x <= bnd and self.c_gcd_int(x, y) == 1:
            n[0] = y
            d[0] = x
            return 0

        n[0] = 0
        d[0] = 0
        return 0

    def rational_recon_int(self, int a, int m):
        """
        Rational reconstruction of a modulo m.
        """
        cdef int n, d
        self.c_rational_recon_int(a, m, &n, &d)
        return (n, d)


# The long long versions are next.
cdef class arith_llong:

    cdef long long abs_longlong(self, long long x) except -1:
        if x < 0:
            return -x
        return x

    cdef long long sign_longlong(self, long long n) except -2:
        if n < 0:
            return -1
        return 1

    cdef long long c_gcd_longlong(self, long long a, long long b) except -1:
        cdef long long c
        if a == 0:
            return self.abs_longlong(b)
        if b == 0:
            return self.abs_longlong(a)
        if a < 0:
            a = -a
        if b < 0:
            b = -b
        while b:
            c = a % b
            a = b
            b = c
        return a

    def gcd_longlong(self, long long a, long long b):
        return self.c_gcd_longlong(a, b)

    cdef long long c_xgcd_longlong(self, long long a, long long b,
                                   long long *ss,
                                   long long *tt) except -1:
        cdef long long psign, qsign, p, q, r, s, c, quot, new_r, new_s

        if a == 0:
            ss[0] = 0
            tt[0] = self.sign_longlong(b)
            return self.abs_longlong(b)

        if b == 0:
            ss[0] = self.sign_longlong(a)
            tt[0] = 0
            return self.abs_longlong(a)

        psign = 1
        qsign = 1

        if a < 0:
            a = -a
            psign = -1
        if b < 0:
            b = -b
            qsign = -1

        p = 1
        q = 0
        r = 0
        s = 1
        while b:
            c = a % b
            quot = a / b
            a = b
            b = c
            new_r = p - quot * r
            new_s = q - quot * s
            p = r
            q = s
            r = new_r
            s = new_s

        ss[0] = p * psign
        tt[0] = q * qsign

        return a

    cdef long long c_inverse_mod_longlong(self, long long a, long long m) except -1:
        cdef long long g, s, t
        g = self.c_xgcd_longlong(a, m, &s, &t)
        if g != 1:
            raise ArithmeticError("The inverse of %s modulo %s is not defined." % (a, m))
        s = s % m
        if s < 0:
            s = s + m
        return s

    def inverse_mod_longlong(self, long long a, long long m):
        return self.c_inverse_mod_longlong(a, m)

    cdef long long c_rational_recon_longlong(self, long long a, long long m,
                                             long long *n, long long *d) except -1:
        cdef long long u, v, u0, u1, u2, v0, v1, v2, q, t0, t1, t2, x, y
        cdef float bnd

        if m > 2147483647:
            raise OverflowError(f"The modulus m(={m}) must be at most 2147483647")

        a = a % m

        if a == 0 or m == 0:
            n[0] = 0
            d[0] = 1
            return 0

        if m < 0:
            m = -m
        if a < 0:
            a = m - a
        if a == 1:
            n[0] = 1
            d[0] = 1
            return 0

        u = m
        v = a
        bnd = sqrt(m/2.0)
        u0 = 1
        u1 = 0
        u2 = u
        v0 = 0
        v1 = 1
        v2 = v
        while self.abs_longlong(v2) > bnd:
            q = u2 / v2   # floor is implicit
            t0 = u0 - q * v0
            t1 = u1 - q * v1
            t2 = u2 - q * v2
            u0 = v0
            u1 = v1
            u2 = v2
            v0 = t0
            v1 = t1
            v2 = t2

        x = self.abs_longlong(v1)
        y = v2
        if v1 < 0:
            y = -1*y
        if x <= bnd and self.gcd_longlong(x, y) == 1:
            n[0] = y
            d[0] = x
            return 0

        n[0] = 0
        d[0] = 0
        return 0

    def rational_recon_longlong(self, long long a, long long m):
        """
        Rational reconstruction of a modulo m.
        """
        cdef long long n, d
        self.c_rational_recon_longlong(a, m, &n, &d)
        return (n, d)



# sage_setup: distribution = sagemath-categories
"""
Quotient Ring Elements

AUTHORS:

- William Stein
"""

# ****************************************************************************
#       Copyright (C) 2005 William Stein <wstein@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.structure.element import RingElement
from sage.structure.richcmp import richcmp, rich_to_bool


try:
    from sage.interfaces.singular import singular as singular_default
except ImportError:
    singular_default = None


class QuotientRingElement(RingElement):
    """
    An element of a quotient ring `R/I`.

    INPUT:

    - ``parent`` -- the ring `R/I`

    - ``rep`` -- a representative of the element in `R`; this is used
      as the internal representation of the element

    - ``reduce`` -- boolean (default: ``True``); if ``True``, then the
      internal representation of the element is ``rep`` reduced modulo
      the ideal `I`

    EXAMPLES::

        sage: R.<x> = PolynomialRing(ZZ)
        sage: S.<xbar> = R.quo((4 + 3*x + x^2, 1 + x^2)); S
        Quotient of Univariate Polynomial Ring in x over Integer Ring
         by the ideal (x^2 + 3*x + 4, x^2 + 1)
        sage: v = S.gens(); v
        (xbar,)

    ::

        sage: loads(v[0].dumps()) == v[0]
        True

    ::

        sage: R.<x,y> = PolynomialRing(QQ, 2)
        sage: S = R.quo(x^2 + y^2); S
        Quotient of Multivariate Polynomial Ring in x, y over Rational Field
         by the ideal (x^2 + y^2)
        sage: S.gens()                                                                  # needs sage.libs.singular
        (xbar, ybar)

    We name each of the generators.

    ::

        sage: # needs sage.libs.singular
        sage: S.<a,b> = R.quotient(x^2 + y^2)
        sage: a
        a
        sage: b
        b
        sage: a^2 + b^2 == 0
        True
        sage: b.lift()
        y
        sage: (a^3 + b^2).lift()
        -x*y^2 + y^2
    """
    def __init__(self, parent, rep, reduce=True):
        """
        An element of a quotient ring `R/I`.  See
        ``QuotientRingElement`` for full documentation.

        EXAMPLES::

            sage: R.<x> = PolynomialRing(ZZ)
            sage: S.<xbar> = R.quo((4 + 3*x + x^2, 1 + x^2)); S
            Quotient of Univariate Polynomial Ring in x over Integer Ring
             by the ideal (x^2 + 3*x + 4, x^2 + 1)
            sage: v = S.gens(); v
            (xbar,)
        """
        RingElement.__init__(self, parent)
        self.__rep = rep
        if reduce:
            self._reduce_()

    def _reduce_(self):
        """
        Reduce the element modulo the defining ideal of the quotient
        ring.  This internal method replaces the cached representative
        by one in reduced form.

        (Note that this has nothing to do with pickling.)

        TESTS::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: a._reduce_()                                                          # needs sage.libs.singular
            sage: a._QuotientRingElement__rep                                           # needs sage.libs.singular
            x
        """
        I = self.parent().defining_ideal()
        self.__rep = I.reduce(self.__rep)

    def lift(self):
        """
        If ``self`` is an element of `R/I`, then return ``self`` as an
        element of `R`.

        EXAMPLES::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: a.lift()                                                              # needs sage.libs.singular
            x
            sage: (3/5*(a + a^2 + b^2)).lift()                                          # needs sage.libs.singular
            3/5*x
        """
        return self.__rep

    def __bool__(self):
        """
        Return ``True`` if quotient ring element is nonzero in the
        quotient ring `R/I`, by determining whether the element
        is in `I`.

        EXAMPLES::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: bool(a)     # indirect doctest                                        # needs sage.libs.singular
            True
            sage: bool(S(0))                                                            # needs sage.libs.singular
            False

        TESTS::

            sage: bool(a - a)                                                           # needs sage.libs.singular
            False
        """
        return self.__rep not in self.parent().defining_ideal()

    def is_unit(self):
        """
        Return ``True`` if ``self`` is a unit in the quotient ring.

        EXAMPLES::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(1 - x*y); type(a)                     # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: a*b                                                                   # needs sage.libs.singular
            1
            sage: S(2).is_unit()                                                        # needs sage.libs.singular
            True

        Check that :issue:`29469` is fixed::

            sage: a.is_unit()                                                           # needs sage.libs.singular
            True
            sage: (a+b).is_unit()                                                       # needs sage.libs.singular
            False
        """
        if self.__rep.is_unit():
            return True
        from sage.categories.fields import Fields
        if self.parent() in Fields():
            return not self.is_zero()
        try:
            self.__invert__()
            return True
        except ArithmeticError:
            return False
        raise NotImplementedError

    def _repr_(self):
        """
        String representation.

        TESTS::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: a-2*a*b     # indirect doctest                                        # needs sage.libs.singular
            -2*a*b + a

        In :issue:`11068`, the case of quotient rings without
        assigned names has been covered as well::

            sage: # needs sage.libs.singular
            sage: S = SteenrodAlgebra(2)
            sage: I = S * [S.0 + S.1] * S
            sage: Q = S.quo(I)
            sage: Q.0
            Sq(1)
        """
        from sage.structure.parent_gens import localvars
        P = self.parent()
        R = P.cover_ring()
        # We print by temporarily (and safely!) changing the variable
        # names of the covering structure R to those of P.
        # These names get changed back, since we're using "with".
        # However, it may occur that no variable names are assigned.
        # That holds, in particular, if there are infinitely many
        # generators, as for Steenrod algebras.
        try:
            P.variable_names()
        except ValueError:
            return str(self.__rep)
        with localvars(R, P.variable_names(), normalize=False):
            return str(self.__rep)

    def _latex_(self):
        """
        Return the LaTeX representation as a string.

        EXAMPLES::

            sage: R = PolynomialRing(QQ, 'a, b, c')
            sage: a = R.gen(0)
            sage: I = R.ideal(a**2 + a + 1)
            sage: S = R.quotient(I, names=R.variable_names())
            sage: a = S.gen(0)                                                          # needs sage.libs.singular
            sage: latex(a)
            a
        """
        from sage.structure.parent_gens import localvars
        P = self.parent()
        R = P.cover_ring()
        # see _repr_ above for the idea
        try:
            P.variable_names()
        except ValueError:
            return self.__rep._latex_()
        with localvars(R, P.variable_names(), normalize=False):
            return self.__rep._latex_()

    def __pari__(self):
        """
        The Pari representation of this quotient element.

        Since Pari does not support quotients by non-principal ideals,
        this function will raise an error in that case.

        EXAMPLES::

            sage: R.<x,y> = QQ[]
            sage: I = R.ideal(x^3, y^3)
            sage: S.<xb,yb> = R.quo(I)                                                  # needs sage.libs.singular
            sage: pari(xb)                                                              # needs sage.libs.pari sage.libs.singular
            Traceback (most recent call last):
            ...
            ValueError: Pari does not support quotients by non-principal ideals

        Note that the quotient does work in the case that the ideal is principal::

            sage: I = R.ideal(x^3 + y^3)
            sage: S.<xb,yb> = R.quo(I)                                                  # needs sage.libs.singular
            sage: pari(xb)^4                                                            # needs sage.libs.pari sage.libs.singular
            Mod(-y^3*x, x^3 + y^3)
            sage: pari(yb)^4                                                            # needs sage.libs.pari sage.libs.singular
            Mod(y^4, x^3 + y^3)
        """
        gens = self.parent().defining_ideal().gens()
        if len(gens) != 1:
            raise ValueError("Pari does not support quotients by non-principal ideals")
        return self.__rep.__pari__().Mod(gens[0])

    def _add_(self, right):
        """
        Add quotient ring element ``self`` to another quotient ring
        element, ``right``. If the quotient is `R/I`, the addition is
        carried out in `R` and then reduced to `R/I`.

        EXAMPLES::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: a + b                                                                 # needs sage.libs.singular
            a + b

        TESTS::

            sage: a._add_(b)                                                            # needs sage.libs.singular
            a + b
        """
        return self.__class__(self.parent(), self.__rep + right.__rep)

    def _sub_(self, right):
        """
        Subtract quotient ring element ``right`` from quotient ring
        element ``self``. If the quotient is `R/I`, the subtraction is
        carried out in `R` and then reduced to `R/I`.

        EXAMPLES::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: a - b                                                                 # needs sage.libs.singular
            a - b

        TESTS::

            sage: a._sub_(b)                                                            # needs sage.libs.singular
            a - b
        """
        return self.__class__(self.parent(), self.__rep - right.__rep)

    def _mul_(self, right):
        """
        Multiply quotient ring element ``self`` by another quotient ring
        element, ``right``. If the quotient is `R/I`, the multiplication is
        carried out in `R` and then reduced to `R/I`.

        EXAMPLES::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: a * b                                                                 # needs sage.libs.singular
            a*b

        TESTS::

            sage: a._mul_(b)                                                            # needs sage.libs.singular
            a*b
            sage: a._mul_(a)                                                            # needs sage.libs.singular
            -b^2
        """
        return self.__class__(self.parent(), self.__rep * right.__rep)

    def _div_(self, right):
        """
        Divide quotient ring element ``self`` by another quotient ring
        element, ``right``. If the quotient is `R/I`, the division is
        carried out in `R` and then reduced to `R/I`.

        EXAMPLES::

            sage: R.<x,y> = QQ[]
            sage: I = R.ideal([x^2 + 1, y^3 - 2])
            sage: S.<i,cuberoot> = R.quotient(I)                                        # needs sage.libs.singular
            sage: 1/(1+i)                                                               # needs sage.libs.singular
            -1/2*i + 1/2

        Confirm via symbolic computation::

            sage: 1/(1+sqrt(-1))                                                        # needs sage.symbolic
            -1/2*I + 1/2

        Another more complicated quotient::

            sage: b = 1/(i+cuberoot); b                                                 # needs sage.libs.singular
            1/5*i*cuberoot^2 - 2/5*i*cuberoot + 2/5*cuberoot^2 - 1/5*i + 1/5*cuberoot - 2/5
            sage: b*(i+cuberoot)                                                        # needs sage.libs.singular
            1


        Another really easy example::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: a / S(2)                                                              # needs sage.libs.singular
            1/2*a
            sage: (a*b)._div_(b)                                                        # needs sage.libs.singular
            a

        An example in which we try to divide in a ring that is not a
        field::

            sage: R.<x,y> = QQ[]
            sage: I = R.ideal([x^2 - 1, y^3 - 2])
            sage: S.<a,cuberoot> = R.quotient(I)                                        # needs sage.libs.singular
            sage: 1/cuberoot                                                            # needs sage.libs.singular
            1/2*cuberoot^2
            sage: 1/a                                                                   # needs sage.libs.singular
            a

        Check that :issue:`13670` is fixed (i.e. that the error message
        actually describes what happens when the result of division is not defined)::

            sage: R.<x1,x2> = QQ[]
            sage: S = R.quotient_ring( R.ideal(x2**2 + x1 - 2, x1**2 - 1) )
            sage: 1 / S(x1 + x2)                                                        # needs sage.libs.singular
            Traceback (most recent call last):
            ...
            ArithmeticError: Division failed. The numerator is not a multiple of the denominator.

        An example over a polynomial ring over a polynomial ring,
        which doesn't work (yet; obviously, this could be made to work
        by converting to a single polynomial quotient ring
        internally)::

            sage: R.<x> = QQ[]
            sage: S.<y,z> = R[]
            sage: Z.<ybar,zbar> = S.quotient([y^2 - 2, z^2 - 3])                        # needs sage.libs.singular
            Traceback (most recent call last):
            ...
            TypeError: Can only reduce polynomials over fields.
        """
        # Special case: if self==0 (and right is nonzero), just return self.
        if not self:
            if not right:
                raise ZeroDivisionError
            return self

        # We are computing L/R modulo the ideal.
        (L, R) = (self.__rep, right.__rep)
        P = self.parent()
        I = P.defining_ideal()

        if not hasattr(I, 'groebner_basis'):
            # Try something very naive -- somebody will improve this
            # in the future.
            try:
                return L * R.inverse_mod(I)
            except NotImplementedError:
                if R.is_unit():
                    return L * ~R
                else:
                    raise

        # Now the parent is a polynomial ring, so we have an algorithm
        # at our disposal.

        # Our algorithm is to write L in terms of R and a Groebner
        # basis for the defining ideal.  We compute a Groebner basis
        # here explicitly purely for efficiency reasons, since it
        # makes the implicit Groebner basis computation of [R]+B
        # that is done in the lift command below faster.

        B = I.groebner_basis()
        try:
            XY = L.lift((R,) + tuple(B))
        except ValueError:
            raise ArithmeticError("Division failed. The numerator is not "
                                  "a multiple of the denominator.")
        return P(XY[0])

    def _im_gens_(self, codomain, im_gens, base_map=None):
        """
        Return the image of ``self`` in ``codomain`` under the map
        that sends ``self.parent().gens()`` to ``im_gens``.

        INPUT:

        - ``codomain`` -- a ring

        - ``im_gens`` -- tuple of elements `f(x)` in ``codomain``,
          one for each `x` in ``self.parent().gens()``, that define
          a homomorphism `f` from ``self.parent()`` to ``codomain``

        OUTPUT:

        The image of ``self`` in ``codomain`` under the above
        homomorphism `f`.

        EXAMPLES:

        Ring homomorphisms whose domain is the fraction field of a
        quotient ring work correctly (see :issue:`16135`)::

            sage: # needs sage.libs.singular
            sage: R.<x, y> = QQ[]
            sage: K = R.quotient(x^2 - y^3).fraction_field()
            sage: L.<t> = FunctionField(QQ)
            sage: f = K.hom((t^3, t^2))
            sage: list(map(f, K.gens()))
            [t^3, t^2]
            sage: xbar, ybar = K.gens()
            sage: f(1/ybar)
            1/t^2
            sage: f(xbar/ybar)
            t
        """
        return self.lift()._im_gens_(codomain, im_gens, base_map=base_map)

    def __int__(self):
        """
        Try to convert ``self`` (an element of `R/I`) to an integer by
        converting its lift in `R` to an integer.  Return a TypeError
        if no such conversion can be found.

        EXAMPLES::

            sage: # needs sage.libs.singular
            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: int(S(-3))                # indirect doctest
            -3
            sage: type(int(S(-3)))
            <... 'int'>
            sage: int(a)
            Traceback (most recent call last):
            ...
            TypeError: unable to convert non-constant polynomial x to <class 'int'>
        """
        return int(self.lift())

    def _integer_(self, Z):
        """
        EXAMPLES::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: ZZ(S(-3))                                                             # needs sage.libs.singular
            -3

        TESTS::

            sage: type(ZZ(S(-3)))                                                       # needs sage.libs.singular
            <class 'sage.rings.integer.Integer'>
        """
        return Z(self.lift())

    def _rational_(self):
        """
        EXAMPLES::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: QQ(S(-2/3))                                                           # needs sage.libs.singular
            -2/3

        TESTS::

            sage: type(S(-2/3)._rational_())                                            # needs sage.libs.singular
            <class 'sage.rings.rational.Rational'>
        """
        from sage.rings.rational_field import QQ
        return QQ(self.lift())

    def __neg__(self):
        """
        EXAMPLES::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: -a                     # indirect doctest                             # needs sage.libs.singular
            -a
            sage: -(a+b)                                                                # needs sage.libs.singular
            -a - b
        """
        return self.__class__(self.parent(), -self.__rep)

    def __pos__(self):
        """
        TESTS::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: (a+b).__pos__()                                                       # needs sage.libs.singular
            a + b
            sage: c = a+b; c.__pos__() is c                                             # needs sage.libs.singular
            True
        """
        return self

    def __invert__(self):
        """
        EXAMPLES::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: ~S(2/3)                                                               # needs sage.libs.singular
            3/2

        TESTS::

            sage: S(2/3).__invert__()                                                   # needs sage.libs.singular
            3/2

        Note that a is not invertible as an element of R::

            sage: a.__invert__()                                                        # needs sage.libs.singular
            Traceback (most recent call last):
            ...
            ArithmeticError: element is non-invertible
        """
        try:
            inv = self.__rep.inverse_mod(self.parent().defining_ideal())
        except NotImplementedError:
            return self.parent().one()/self
        return self.__class__(self.parent(), inv)

    def __float__(self):
        """
        EXAMPLES::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: float(S(2/3))                                                         # needs sage.libs.singular
            0.6666666666666666
            sage: float(a)                                                              # needs sage.libs.singular
            Traceback (most recent call last):
            ...
            TypeError: unable to convert non-constant polynomial x to <class 'float'>
        """
        return float(self.lift())

    def __hash__(self):
        r"""
        TESTS::

            sage: # needs sage.libs.singular
            sage: R.<x,y> = QQ[]
            sage: S.<a,b> = R.quo(x^2 + y^2)
            sage: c = a*a + b
            sage: hash(a) != hash(b)
            True
        """
        return hash(self.__rep)

    def _richcmp_(self, other, op):
        """
        EXAMPLES::

            sage: # needs sage.libs.singular
            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: a > b    # indirect doctest
            True
            sage: b > a
            False
            sage: a == loads(dumps(a))
            True

        TESTS::

            sage: a == (a+1-1)                                                          # needs sage.libs.singular
            True
            sage: a > b                                                                 # needs sage.libs.singular
            True

        See :issue:`7797`::

            sage: # needs sage.combinat sage.libs.singular sage.modules
            sage: F.<x,y,z> = FreeAlgebra(QQ, implementation='letterplace')
            sage: I = F * [x*y + y*z, x^2 + x*y - y*x - y^2] * F
            sage: Q = F.quo(I)
            sage: Q.0^4    # indirect doctest
            ybar*zbar*zbar*xbar + ybar*zbar*zbar*ybar + ybar*zbar*zbar*zbar

        The issue from :issue:`8005` was most likely fixed as part of
        :issue:`9138`::

            sage: # needs sage.libs.singular
            sage: F = GF(5)
            sage: R.<x,y> = F[]
            sage: I = Ideal(R, [x, y])
            sage: S.<x1,y1> = QuotientRing(R, I)
            sage: x1^4
            0
        """
        # A containment test is not implemented for univariate polynomial
        # ideals. There are cases in which one would not like to add
        # elements of different degrees. The whole quotient stuff relies
        # in I.reduce(x) returning a normal form of x with respect to I.
        # Hence, we will not use more than that.

        # Since we have to compute normal forms anyway, it makes sense
        # to use it for comparison in the case of an inequality as well.
        if self.__rep == other.__rep:
            # Use a shortpath, so that we avoid expensive reductions
            return rich_to_bool(op, 0)
        I = self.parent().defining_ideal()
        return richcmp(I.reduce(self.__rep), I.reduce(other.__rep), op)

    def lt(self):
        """
        Return the leading term of this quotient ring element.

        EXAMPLES::

            sage: # needs sage.libs.singular
            sage: R.<x,y,z> = PolynomialRing(GF(7), 3, order='lex')
            sage: I = sage.rings.ideal.FieldIdeal(R)
            sage: Q = R.quo(I)
            sage: f = Q(z*y + 2*x)
            sage: f.lt()
            2*xbar

        TESTS::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: (a + 3*a*b + b).lt()                                                  # needs sage.libs.singular
            3*a*b
        """
        return self.__class__(self.parent(), self.__rep.lt())

    def lm(self):
        """
        Return the leading monomial of this quotient ring element.

        EXAMPLES::

            sage: # needs sage.libs.singular
            sage: R.<x,y,z> = PolynomialRing(GF(7), 3, order='lex')
            sage: I = sage.rings.ideal.FieldIdeal(R)
            sage: Q = R.quo(I)
            sage: f = Q(z*y + 2*x)
            sage: f.lm()
            xbar

        TESTS::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: (a+3*a*b+b).lm()                                                      # needs sage.libs.singular
            a*b
        """
        return self.__class__(self.parent(), self.__rep.lm())

    def lc(self):
        """
        Return the leading coefficient of this quotient ring element.

        EXAMPLES::

            sage: # needs sage.libs.singular
            sage: R.<x,y,z> = PolynomialRing(GF(7), 3, order='lex')
            sage: I = sage.rings.ideal.FieldIdeal(R)
            sage: Q = R.quo(I)
            sage: f = Q(z*y + 2*x)
            sage: f.lc()
            2

        TESTS::

            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)                   # needs sage.libs.singular
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: (a + 3*a*b + b).lc()                                                  # needs sage.libs.singular
            3
        """
        return self.__rep.lc()

    def variables(self):
        """
        Return all variables occurring in ``self``.

        OUTPUT:

        A tuple of linear monomials, one for each variable occurring
        in ``self``.

        EXAMPLES::

            sage: # needs sage.libs.singular
            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: a.variables()
            (a,)
            sage: b.variables()
            (b,)
            sage: s = a^2 + b^2 + 1; s
            1
            sage: s.variables()
            ()
            sage: (a + b).variables()
            (a, b)
        """
        return tuple(self.__class__(self.parent(), v) for v in self.__rep.variables())

    def monomials(self):
        """
        Return the monomials in ``self``.

        OUTPUT: list of monomials

        EXAMPLES::

            sage: # needs sage.libs.singular
            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: a.monomials()
            [a]
            sage: (a + a*b).monomials()
            [a*b, a]
            sage: R.zero().monomials()
            []
        """
        return [self.__class__(self.parent(), m) for m in self.__rep.monomials()]

    def _singular_(self, singular=None):
        """
        Return Singular representation of ``self``.

        INPUT:

        - ``singular`` -- a non-standard interpreter may be provided

        EXAMPLES::

            sage: # needs sage.libs.singular
            sage: P.<x,y> = PolynomialRing(GF(2), 2)
            sage: I = sage.rings.ideal.FieldIdeal(P)
            sage: Q = P.quo(I)
            sage: Q._singular_()
            polynomial ring, over a field, global ordering
            //   coefficients: ZZ/2
            //   number of vars : 2
            //        block   1 : ordering dp
            //                  : names    x y
            //        block   2 : ordering C
            // quotient ring from ideal
            _[1]=x2+x
            _[2]=y2+y
            sage: xbar = Q(x); xbar
            xbar
            sage: xbar._singular_()
            x
            sage: Q(xbar._singular_()) # a round-trip
            xbar

        TESTS::

            sage: # needs sage.libs.singular
            sage: R.<x,y> = QQ[]; S.<a,b> = R.quo(x^2 + y^2); type(a)
            <class 'sage.rings.quotient_ring.QuotientRing_generic_with_category.element_class'>
            sage: (a - 2/3*b)._singular_()
            x-2/3*y
            sage: S((a - 2/3*b)._singular_())
            a - 2/3*b
        """
        if singular is None:
            singular = singular_default
            if singular is None:
                raise ImportError("could not import singular")
        return self.__rep._singular_(singular)

    def _magma_init_(self, magma):
        """
        Return the Magma representation of this quotient ring element.

        EXAMPLES::

            sage: # needs sage.libs.singular
            sage: P.<x,y> = PolynomialRing(GF(2))
            sage: Q = P.quotient(sage.rings.ideal.FieldIdeal(P))
            sage: xbar, ybar = Q.gens()
            sage: magma(xbar)                   # optional - magma
            x
            sage: xbar._magma_init_(magma)      # optional - magma
            '_sage_[...]!_sage_ref...'
        """
        g = magma(self.__rep)
        R = magma(self.parent())
        return '{}!{}'.format(R.name(), g._ref())

    def _macaulay2_(self, macaulay2=None):
        """
        The Macaulay2 element corresponding to this polynomial.

        EXAMPLES::

            sage: # needs sage.libs.singular
            sage: R.<x,y> = PolynomialRing(GF(7), 2)
            sage: Q = R.quotient([x^2 - y])
            sage: x, y = Q.gens()
            sage: f = (x^3 + 2*y^2*x)^7; f
            2*xbar*ybar^17 + xbar*ybar^10
            sage: mf = macaulay2(f); mf         # optional - macaulay2
                17      10
            2x*y   + x*y
            sage: mf.sage()                     # optional - macaulay2
            2*x*y^17 + x*y^10
            sage: mf.sage() == f                # optional - macaulay2
            True
            sage: Q(mf)                         # optional - macaulay2
            2*xbar*ybar^17 + xbar*ybar^10

        In Macaulay2, the variable names for a quotient ring are inherited from
        the variable names of the ambient ring. This is in contrast to Sage's
        default behaviour in which the variable names for the quotient ring are
        obtained by appending ``bar`` to the variable names of the ambient
        ring. This can be controlled using the ``names`` argument of the
        ``quotient`` method.

        ::

            sage: # needs sage.libs.singular
            sage: R.<x,y> = PolynomialRing(GF(7), 2)
            sage: Q = R.quotient([x^2 - y], names=R.gens())
            sage: x, y = Q.gens()
            sage: f = (x^3 + 2*y^2*x)^7; f
            2*x*y^17 + x*y^10
            sage: macaulay2(f)                  # optional - macaulay2
                17      10
            2x*y   + x*y
            sage: _.sage()                      # optional - macaulay2
            2*x*y^17 + x*y^10

        TESTS:

        Check that changing the currently defined global variables (`x`, `y`,
        ...) in Macaulay2 does not affect the result of this conversion::

            sage: # needs sage.libs.singular
            sage: R.<x,y> = PolynomialRing(GF(7), 2)
            sage: Q = R.quotient([x^2 - y], names=R.gens())
            sage: x, y = Q.gens()
            sage: f = (x^3 + 2*y^2*x)^7
            sage: macaulay2(f)                  # optional - macaulay2
                17      10
            2x*y   + x*y
            sage: macaulay2.use(R.quotient([x, y]))     # optional - macaulay2
            sage: macaulay2(f)                  # optional - macaulay2
                17      10
            2x*y   + x*y
        """
        if macaulay2 is None:
            from sage.interfaces.macaulay2 import macaulay2 as m2_default
            macaulay2 = m2_default
        m2_parent = self.parent()._macaulay2_(macaulay2)
        macaulay2.use(m2_parent)
        return macaulay2.substitute(repr(self.lift()), m2_parent)

    def reduce(self, G):
        r"""
        Reduce this quotient ring element by a set of quotient ring
        elements ``G``.

        INPUT:

        - ``G`` -- list of quotient ring elements

        .. WARNING::

            This method is not guaranteed to return unique minimal results.
            For quotients of polynomial rings, use
            :meth:`~sage.rings.polynomial.multi_polynomial_ideal.MPolynomialIdeal.reduce`
            on the ideal generated by ``G``, instead.

        EXAMPLES::

            sage: # needs sage.libs.singular
            sage: P.<a,b,c,d,e> = PolynomialRing(GF(2), 5, order='lex')
            sage: I1 = ideal([a*b + c*d + 1, a*c*e + d*e,
            ....:             a*b*e + c*e, b*c + c*d*e + 1])
            sage: Q = P.quotient(sage.rings.ideal.FieldIdeal(P))
            sage: I2 = ideal([Q(f) for f in I1.gens()])
            sage: f = Q((a*b + c*d + 1)^2  + e)
            sage: f.reduce(I2.gens())
            ebar

        Notice that the result above is not minimal::

            sage: I2.reduce(f)                                                          # needs sage.libs.singular
            0
        """
        try:
            G = [f.lift() for f in G]
        except TypeError:
            pass
        # reduction w.r.t. the defining ideal is performed in the
        # constructor
        return self.__class__(self.parent(), self.__rep.reduce(G))
